@@ -103,8 +103,7 @@ macro_rules! impl_command_pool {
         #[doc = $doc]
         #[derive(Debug)]
         pub struct $pool_name {
-            /// The raw Vulkan command pool.
-            pub raw: vk::CommandPool,
+            raw: vk::CommandPool,
             queue_type: QueueType,
             command_buffer_counter: u64,
             context: Arc<Context>,
@@ -147,6 +146,12 @@ macro_rules! impl_command_pool {
                 })
             }
 
+            /// The raw Vulkan command pool handle.
+            #[inline]
+            pub fn raw(&self) -> vk::CommandPool {
+                self.raw
+            }
+
             /// Creates a new command buffer.
             pub fn create_command_buffer(
                 &mut self,
@@ -234,8 +239,7 @@ macro_rules! impl_command_buffer {
         #[doc = $doc]
         #[derive(Debug)]
         pub struct $buffer_name {
-            /// The raw Vulkan command buffer.
-            pub(crate) raw: vk::CommandBuffer,
+            raw: vk::CommandBuffer,
             pub(crate) wait_semaphores: Vec<CommandBufferSemaphoreInner>,
             pub(crate) signal_semaphores: Vec<CommandBufferSemaphoreInner>,
             pool: vk::CommandPool,
@@ -259,6 +263,12 @@ macro_rules! impl_command_buffer {
                 }
             }
 
+            /// The raw Vulkan command buffer handle.
+            #[inline]
+            pub fn raw(&self) -> vk::CommandBuffer {
+                self.raw
+            }
+
             /// Begins to record the command buffer. Encoder will finish recording on drop.
             pub fn record(&self) -> Result<$encoder_name> {
                 let encoder = $encoder_name {
